// Copyright (C) 2021 The Qt Company Ltd.
// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
// Qt-Security score:insignificant

#ifndef QTJSONRPCGLOBAL_H
#define QTJSONRPCGLOBAL_H

#include <QtCore/qglobal.h>

// include qtjsonrpcexports.h here instead
// if module is no longer unconditionally static
#define Q_JSONRPC_EXPORT

#endif // QTJSONRPCGLOBAL_H
