/* This file was generated by syncqt. */
#ifndef QT_QTLANGUAGESERVER_VERSION_H
#define QT_QTLANGUAGESERVER_VERSION_H

#define QTLANGUAGESERVER_VERSION_STR "6.11.2"

#define QTLANGUAGESERVER_VERSION 0x0601102

#endif // QT_QTLANGUAGESERVER_VERSION_H
