/* This file was generated by syncqt. */
#ifndef QT_QTJSONRPC_VERSION_H
#define QT_QTJSONRPC_VERSION_H

#define QTJSONRPC_VERSION_STR "6.11.2"

#define QTJSONRPC_VERSION 0x0601102

#endif // QT_QTJSONRPC_VERSION_H
